﻿/* 
==============================================================================
	RotateBox demonstrates how to rotate a Box with a Transform3D in Sandy
===============================================================================
*/
import sandy.core.data.*;
import sandy.core.group.*;
import sandy.primitive.*;
import sandy.view.*;
import sandy.core.*;
import sandy.skin.*;
import sandy.util.*;
import sandy.core.transform.*;
import sandy.events.*;

var screen:ClipScreen;	// Presentation surface
var world:World3D = World3D.getInstance(); // Our 3D world
var rotation:Transform3D; // Our rotation transform

function init( Void ):Void {
	// Create a clipScreen as presentation surface for the World3D to draw on
	screen = new ClipScreen( this.createEmptyMovieClip('screen', 1), 200, 200 );
	// Create a camera, so we can see our world. 
	// Give it a focal distanse ( equivalent ) and a reference to the viewing screen.
	var cam:Camera3D = new Camera3D( 800, screen );
	// Position the camera at a negative distance along the z axis
	// By default it is looking at the origin of the world coordinate system [0,0,0]
	cam.setPosition(0,0,-500);
	// Attach the camera to the world
	world.addCamera( cam );
	// Create the root group, the banch node for all objects in our world
	var bg:Group = new Group();
	// and attach it to the world. this node is responisble 
	// for drawing all its ( possibly transformed child nodes
	world.setRootGroup( bg );
	// Create the scen and attach it to the rootGroup
	createScene( bg );
	// Create the global coordinate system for reference, only axes
	createCoordinateSystem( bg, false, 1 );
	// Listen to the rotation button
	rotateButton.onRelease = rotate;
	// Finally we start rendering the world
	world.render();
}
function rotate(){
	rotation.rot(Number(xStep.text),Number(yStep.text),Number(zStep.text));	
}
function createScene( bg:Group ):Void {
	// Start by creating a Box primitive. By setting all edges to 50 pixels, we'll get a cube
	var cube:Object3D = new Box( 50, 50, 50, 'quad' );
	// We can create a skin to make the cube look a little better
	// In this case we use a mixed skin with a semi transparent red color and black thin lines
	var skin:Skin = new MixedSkin( 0xF28F35, 100, 0, 0, 0 );
	// Enable the default light source on the skin
	skin.setLightingEnable( true )
	// Apply the skin to the cube.
	cube.setSkin( skin );
	// First we create a TransformationGroup to fascilitate the rotation
	var tg:TransformGroup = new TransformGroup();
	// We create Transform3D to carry out the transformation in 3 dimensions
	rotation = new Transform3D();
	// We have to tell the Transformation what to do.
	// We want it to rotate our cube by different amounts around the global axes
	rotation.rot(20,30,0);
	// The transform is set in the transform group which will hold our cube
	tg.setTransform( rotation );
	// Our cube is added to the transform group
	tg.addChild( cube );
	// And finally the transformgroup is added as a child to the rootGroup
	bg.addChild( tg );
}
// Create a coordinate system for reference
function createCoordinateSystem( bg:Group, doPlanes:Boolean, grid:Number ):Void {
	// Create the coordinate axes
	var xAxis:Object3D = new Line3D( new Vector(-150,0,0), new Vector(150,0,0) );
	var yAxis:Object3D = new Line3D( new Vector(0,-150,0), new Vector(0,150,0) );
	var zAxis:Object3D = new Line3D( new Vector(0,0,-150), new Vector(0,0,150) );
	// Give the axes distinct colors
	xAxis.setSkin( new SimpleLineSkin( 1, 0xff0000, 60 ) ); // red
	yAxis.setSkin( new SimpleLineSkin( 1, 0x00ff00, 60 ) ); // green
	zAxis.setSkin( new SimpleLineSkin( 1, 0x0000ff, 60 ) ); // blue
	// Create the axes group
	var axes:Group = new Group();
	axes.addChild(xAxis);
	axes.addChild(yAxis);
	axes.addChild(zAxis);
	// Add the coordinate axes to the rootGroup
	bg.addChild( axes );

	// Create the coordiante planes if so ordered
	if( doPlanes){
		
		var xyPlane:Object3D = new Plane3D(100,100,grid,'quad');
		xyPlane.setSkin( new MixedSkin(0x00FF00, 10, 0, 15, 1));
	
		var yzPlane:Object3D = new Plane3D(100,100,grid,'quad');
		yzPlane.setSkin( new MixedSkin(0x0000FF, 10, 0, 15, 1));

		var zxPlane:Object3D = new Plane3D(100,100,grid,'quad');
		zxPlane.setSkin( new MixedSkin(0xFF0000, 10, 0, 15, 1));

		// Create rotations for the xy plane and the yz plane
		// The zx plane doesn't need any transformation
		var tg1:TransformGroup = new TransformGroup();
		var rot1:Transform3D = new Transform3D();
		rot1.rot(90,0,0);
		tg1.setTransform( rot1 );
		tg1.addChild( xyPlane );
	
		var tg2:TransformGroup = new TransformGroup();
		var rot2:Transform3D = new Transform3D();
		rot2.rot(0,0,90);
		tg2.setTransform( rot2 );
		tg2.addChild( yzPlane );
		
		// Add the transformed coordinate planes to the rootGroup
		bg.addChild( tg1 );
		bg.addChild( tg2 );
		bg.addChild( zxPlane );
	}
}

// Start creating the world
init();